﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Mynx
{
    public partial class ChildWindow : Form
    {
        public bool IsSelecting { get; set; }

        public ImagePanel ImagePanel
        {
            get { return imagePanel1; }
        }
        
        public ChildWindow(Bitmap bmp, string filename)
        {
            InitializeComponent();
            ImagePanel.Image = bmp;
            ImagePanel.CanvasSize = bmp.Size; 
            
            Text = filename;
            ddlZoomSetting.Text = "100%";
            ImagePanel.Zoom = 1f;
            IsSelecting = false;
            DisplayScrollbar();
            SetScrollbarValues();
            SetImagePanelValues();
            ImagePanel.Invalidate();
        }

        private void ChildWindow_Resize(object sender, EventArgs e)
        {
            DisplayScrollbar();
            SetScrollbarValues();
            SetImagePanelValues();
            ImagePanel.Invalidate();
        }

        private void ChildWindow_MouseDown(object sender, MouseEventArgs e)
        {
            base.OnMouseDown(e);
        }

        private void ddlZoomSetting_DropDownItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            ddlZoomSetting.Text = e.ClickedItem.Text;
            ImagePanel.Zoom = int.Parse(e.ClickedItem.Text.Substring(0,e.ClickedItem.Text.IndexOf("%"))) * 0.01f;
            DisplayScrollbar();
            SetScrollbarValues();
            SetImagePanelValues();
            ImagePanel.Invalidate();
        }

        private void vScrollBar1_Scroll(object sender, ScrollEventArgs e)
        {
            SetImagePanelValues();
            ImagePanel.Invalidate();
        }

        private void hScrollBar1_Scroll(object sender, ScrollEventArgs e)
        {
            SetImagePanelValues();
            ImagePanel.Invalidate();
        }

        private void DisplayScrollbar()
        {
            ImagePanel.viewRectWidth = this.Width - vScrollBar1.Width;
            ImagePanel.viewRectHeight = this.Height - hScrollBar1.Height;

            // If the image is wider than view window, show the HScrollBar and adjust the view window
            if (ImagePanel.viewRectWidth > ImagePanel.CanvasSize.Width * ImagePanel.Zoom)
            {
                hScrollBar1.Visible = false;
                hScrollBar1.SendToBack();
                ImagePanel.viewRectHeight = this.Height - statusStrip1.Height;
            }
            else
            {
                hScrollBar1.Show();
                hScrollBar1.BringToFront();
                ImagePanel.viewRectHeight = this.Height - (hScrollBar1.Height + statusStrip1.Height);
            }

            // If the image is taller than view window, show the VScrollBar and adjust the view window
            if (ImagePanel.viewRectHeight > ImagePanel.CanvasSize.Height * ImagePanel.Zoom)
            {
                vScrollBar1.Visible = false;
                vScrollBar1.SendToBack();
                ImagePanel.viewRectWidth = this.Width;
            }
            else
            {
                vScrollBar1.Show();
                vScrollBar1.BringToFront();
                ImagePanel.viewRectWidth = this.Width - vScrollBar1.Width;
            }

            // Set up scrollbars
            
            hScrollBar1.Location = new Point(0, this.Height - (statusStrip1.Height + hScrollBar1.Height));
            hScrollBar1.Width = ImagePanel.viewRectWidth;
            vScrollBar1.Location = new Point(this.Width - vScrollBar1.Width, 0);
            vScrollBar1.Height = ImagePanel.viewRectHeight;
        }

        private void SetScrollbarValues()
        {
            // Set the Maximum, Minimum, LargeChange and SmallChange properties.
            this.vScrollBar1.Minimum = 0;
            this.hScrollBar1.Minimum = 0;

            // If the offset does not make the Maximum less than zero, set its value. 
            if ((ImagePanel.CanvasSize.Width * ImagePanel.Zoom - ImagePanel.viewRectWidth) > 0)
            {
                this.hScrollBar1.Maximum = (int)(ImagePanel.CanvasSize.Width * ImagePanel.Zoom) - ImagePanel.viewRectWidth;
            }
            // If the VScrollBar is visible, adjust the Maximum of the 
            // HSCrollBar to account for the width of the VScrollBar.  
            if (this.vScrollBar1.Visible)
            {
                this.hScrollBar1.Maximum += this.vScrollBar1.Width;
            }
            this.hScrollBar1.LargeChange = this.hScrollBar1.Maximum / 10;
            this.hScrollBar1.SmallChange = this.hScrollBar1.Maximum / 20;

            // Adjust the Maximum value to make the raw Maximum value 
            // attainable by user interaction.
            this.hScrollBar1.Maximum += this.hScrollBar1.LargeChange;

            // If the offset does not make the Maximum less than zero, set its value.    
            if ((ImagePanel.CanvasSize.Height * ImagePanel.Zoom - ImagePanel.viewRectHeight) > 0)
            {
                this.vScrollBar1.Maximum = (int)(ImagePanel.CanvasSize.Height * ImagePanel.Zoom) - ImagePanel.viewRectHeight;
            }

            // If the HScrollBar is visible, adjust the Maximum of the 
            // VSCrollBar to account for the width of the HScrollBar.
            if (this.hScrollBar1.Visible)
            {
                this.vScrollBar1.Maximum += this.hScrollBar1.Height;
            }
            this.vScrollBar1.LargeChange = this.vScrollBar1.Maximum / 10;
            this.vScrollBar1.SmallChange = this.vScrollBar1.Maximum / 20;

            // Adjust the Maximum value to make the raw Maximum value 
            // attainable by user interaction.
            this.vScrollBar1.Maximum += this.vScrollBar1.LargeChange;
        }

        private void SetImagePanelValues()
        {
            if (ImagePanel.Image != null)
            {
                Point pt = new Point((int)(hScrollBar1.Value / ImagePanel.Zoom), (int)(vScrollBar1.Value / ImagePanel.Zoom));
                ImagePanel.SourcePoint = pt;
            }
        }

        private void ChildWindow_ResizeEnd(object sender, EventArgs e)
        {
            DisplayScrollbar();
            SetScrollbarValues();
            SetImagePanelValues();
            ImagePanel.Invalidate();
        }
    }
}
