﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.Windows.Forms;

namespace Mynx
{
    public class DragDropHandler
    {
        private Hashtable mapAllowedExtensions = null;

		public delegate void FileDropHandler(string sFileName);
		public event FileDropHandler FileDropped = null;

		public DragDropHandler(Form form, string [] extensions)
		{
			InitializeEvents(form);
            InitializeExtensionMap(extensions);
		}

		private void InitializeEvents(Form form)
		{
			form.AllowDrop = true;
			form.DragEnter += new DragEventHandler(form_DragEnter);
			form.DragLeave += new EventHandler(form_DragLeave);
			form.DragDrop += new DragEventHandler(form_DragDrop);
		}

		private void InitializeExtensionMap(string [] extensions)
		{
			mapAllowedExtensions = new Hashtable();

			if (extensions != null)
			{
				foreach (string sExtension in extensions)
				{
					mapAllowedExtensions.Add(sExtension, null);
				}
			}
		}

		private void form_DragEnter(object sender, DragEventArgs e)
		{
			if (e.Data.GetDataPresent(DataFormats.FileDrop))
			{
				if (TestExtensions(e))
				{
					e.Effect = DragDropEffects.Copy;
				}
				else
				{
					e.Effect = DragDropEffects.None;
				}
			}
			else
			{
				e.Effect = DragDropEffects.Move;
			}
		}

		private bool TestExtensions(DragEventArgs e)
		{
			string [] asFiles = e.Data.GetData(DataFormats.FileDrop) as string[];

			if (asFiles == null)
			{
				return false;
			}
			else
			{
				foreach (string sFileName in asFiles)
				{
					string sExtension = FileHandler.GetExtension(sFileName);

					if (!mapAllowedExtensions.Contains(sExtension))
					{
						return false;
					}
				}

				return true;
			}
		}

		private void form_DragLeave(object sender, EventArgs e)
		{
		}

		private void form_DragDrop(object sender, DragEventArgs e)
		{
			string [] asFiles = e.Data.GetData(DataFormats.FileDrop) as string[];

			if (asFiles != null && FileDropped != null)
			{
				foreach (string sFileName in asFiles)
				{
					FileDropped(sFileName);
				}
			}
		}
    }
}
