﻿using System.Configuration;
using System.Runtime;
using System.Runtime.Serialization;
using System.Diagnostics;
using System.IO;
using System;
using Microsoft.VisualBasic;

namespace Mynx
{
    [Serializable()]
    public class ExceptionHandler : System.Exception
    {
        #region Variables Declarations
        private string _appName;
        private string _appVersion = "0.0";
        private string _methodAction = "Unknown";
        private string _appCulture = "Unknown";
        private string _userName = System.Environment.UserName;
        private string _machineName = System.Environment.MachineName;
        private string _osVersion = System.Environment.OSVersion.ToString();
        private DateTime _errorDate;
        private DateTime _errorTime;
        private string _exceptionType;
        private bool _bInnerException = false;
        private bool _bCulture;
        private bool _bVersion;
        private string _exceptionLevel = string.Empty;
        private string _message;
        private string _stackTrace;
        const string DEFAULT_CONN_STRING = "YourConnString";
        #endregion

        #region Properties

        #region ReadOnly Properties
        #region Exception Type
        public string ExceptionType {
	        get {
		        _exceptionType = GetBaseException();
		        return _exceptionType;
	        }
        }
        #endregion

        #region Application Culture
        public string ApplicationCulture {
	        get {
		        if (_bCulture == false)
		        {
			        GetApplicationAttributes();
		        }
		        return _appCulture;
	        }
        }
        #endregion

        #region Application Version
        public string ApplicationVersion {
	        get {
		        if (_bVersion == false)
		        {
			        GetApplicationAttributes();
		        }
		        return _appVersion;
	        }
        }
        #endregion

        #region Error Date
        public string ErrorDate {
	        get { return _errorDate.ToString("yyyy/MM/dd"); }
        }
        #endregion

        #region Error Time
        public string ErrorTime {
	        get { return _errorTime.ToShortTimeString(); }
        }
        #endregion

        #region Machine Name
        public string MachineName {
	        get { return _machineName; }
        }
        #endregion

        #region Object Method
        public string Method {
	        get { return base.TargetSite.Name; }
        }
        #endregion

        #region OS Version
        public string OSVersion {
	        get { return _osVersion; }
        }
        #endregion

        #region Inner Exception
        public string AppInnerException {
	        get 
            {
		        if (_bInnerException)
		        {
			        return base.InnerException.ToString();
		        }
		        else
		        {
			        return string.Empty;
		        }
	        }
        }
        #endregion

        #region StackTrace
        public override string StackTrace {
	        get {
		        if (_bInnerException == false)
		        {
			        _stackTrace = base.StackTrace;
		        }
		        else
		        {
			        if ((this.InnerException != null))
			        {
				        _stackTrace = this.InnerException.StackTrace;
			        }
			        else
			        {
				        _stackTrace = string.Empty;
			        }
		        }
		        return _stackTrace;
	        }
        }
        #endregion

        #region User Name
        public string UserName {
	        get { return _userName; }
        }
        #endregion
        #endregion

        #region Read/Write Properties
        #region Object Method Action
        public string MethodAction 
        {
	        get { return _methodAction; }
	        set { _methodAction = value; }
        }
        #endregion

        #region Error Level
        public string ErrorLevel 
        {
	        get { return _exceptionLevel; }
	        set {
		        if (!string.IsNullOrEmpty(value))
		        {
			        _exceptionLevel = value;
		        }
	        }
        }
        #endregion
        #endregion

		#endregion

        #region Constructors

        #region Standards

        public ExceptionHandler() : base()
        {
            this.Clear();
        }

        public ExceptionHandler(string message):base(message)
        {
	         this.Clear();
        }

        public ExceptionHandler(string message, SystemException innerException) : base(message, innerException)
        {
            this.Clear();
	        _bInnerException = true;
        }
        #endregion

        #region Customs
        public ExceptionHandler(string methodAction, string message) : base(message)
        {            
	        _methodAction = methodAction;
        }

        public ExceptionHandler(string methodAction, string message, SystemException innerException) : base(message, innerException)
        {
	        _methodAction = methodAction;
	        _bInnerException = true;
        }

        public ExceptionHandler(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
	        _appCulture = info.GetString("ApplicationCulture");
	        _appVersion = info.GetString("ApplicationVersion");
	        _bCulture = info.GetBoolean("ApplicationCultureFlag");
	        _bVersion = info.GetBoolean("ApplicationVersionFlag");
	        _methodAction = info.GetString("MethodAction");
	        _machineName = info.GetString("MachineName");
	        _userName = info.GetString("UserName");
	        _osVersion = info.GetString("OSVersion");
	        _errorDate = info.GetDateTime("ErrorDate");
	        _errorTime = info.GetDateTime("ErrorTime");
	        _exceptionLevel = info.GetString("ExceptionLevel");
	        _appName = info.GetString("ApplicationName");
	        _exceptionType = info.GetString("ExceptionType");
	        _stackTrace = info.GetString("StackTrace");
        }
        #endregion

        #endregion

		#region Methods
        #region GetBaseException
        /// <summary>
        /// Function to retrieve the base exception in string format
        /// </summary>
        /// <returns>The base exception type</returns>
        /// <remarks></remarks>
        public new string GetBaseException()
        {
            return GetBaseException().ToString();
        }
        #endregion


        #region ToString
        public override string ToString()
        {
            return base.ToString() + Environment.NewLine +
            "Source Object : " + this.Source + Environment.NewLine +
            "Source Method : " + this.TargetSite.Name + Environment.NewLine + 
            "Method Action : " + _methodAction;
        }

		        #endregion

        #region GetObjectData()
        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
	        base.GetObjectData(info, context);
	        info.AddValue("ApplicationCulture", _appCulture);
	        info.AddValue("ApplicationVersion", _appVersion);
	        info.AddValue("ApplicationCultureFlag", _bCulture);
	        info.AddValue("ApplicationVersionFlag", _bVersion);
	        info.AddValue("MethodAction", _methodAction);
	        info.AddValue("MachineName", _machineName);
	        info.AddValue("UserName", _userName);
	        info.AddValue("OSVersion", _osVersion);
	        info.AddValue("ErrorDate", _errorDate);
	        info.AddValue("ErrorTime", _errorTime);
	        info.AddValue("ExceptionLevel", _exceptionLevel);
	        info.AddValue("ApplicationName", _appName);
	        info.AddValue("ExceptionType", _exceptionType);
	        info.AddValue("StackTrace", _stackTrace);
        }
        #endregion

        #region GetApplicationAttributes()
        private void GetApplicationAttributes()
        {
	        if (_bInnerException == false)
	        {
		        if ((this.TargetSite != null))
		        {
			        if ((this.TargetSite.ReflectedType != null))
			        {
				        if ((this.TargetSite.ReflectedType.AssemblyQualifiedName != null))
				        {
					        GetVersionCulture(this.TargetSite.ReflectedType.AssemblyQualifiedName);
				        }
			        }
		        }
	        }
	        else
	        {
		        if ((this.InnerException != null))
		        {
			        if ((this.InnerException.TargetSite != null))
			        {
				        if ((this.InnerException.TargetSite.ReflectedType != null))
				        {
					        if ((this.InnerException.TargetSite.ReflectedType.AssemblyQualifiedName != null))
					        {
						        GetVersionCulture(this.InnerException.TargetSite.ReflectedType.AssemblyQualifiedName);
					        }
				        }
			        }
		        }
	        }
	        _errorDate = DateTime.Now;
            _errorTime = DateAndTime.TimeOfDay;
        }
        #endregion

        #region GetVersionCulture
        /// <summary>
        /// Procedure for retrieving the application culture and version
        /// </summary>
        /// <param name="sAssemblyQualifiedName"></param>
        /// <remarks></remarks>
        private void GetVersionCulture(string sAssemblyQualifiedName)
        {
            string[] assembly = Strings.Split(sAssemblyQualifiedName, " ", -1, CompareMethod.Text);

            foreach (string s1 in assembly)
            {
                if (_bVersion == false)
                {
                    //check t make sure a value was returned
                    if (GetAppVersion(s1) != string.Empty && GetAppVersion(s1) != "")
                    {
                        _appVersion = GetAppVersion(s1);
                        _bVersion = true;
                    }
                    else
                    {
                        _appVersion = "Unknown";
                        _bVersion = false;
                    }
                    
                }
                if (_bCulture == false)
                {
                    //check to make sure a value was returned
                    if (GetAppCulture(s1) != string.Empty && GetAppCulture(s1) != "")
                    {
                        _appCulture = GetAppCulture(s1);
                        _bCulture = true;
                    }
                    else
                    {
                        _appCulture = "Unknown";
                        _bCulture = false;
                    }
                }
                if (_bVersion == true & _bCulture == true)
                {
                    break;
                }
            }
        }
        #endregion

        #region GetAppVersion
        public string GetAppVersion(string str)
        {
            //string to hold our return value
            string version = string.Empty;
            if (str.IndexOf("Version", 0) >= 0)
            {
                string[] sttributes = Strings.Split(str, "=", -1, CompareMethod.Text);
                version = sttributes[1];
                if (version.IndexOf(",") >= 0)
                {
                    version = version.Substring(0, version.IndexOf(","));
                }
            }
            return version;
        }
        #endregion

        #region GetAppCulture
        public string GetAppCulture(string str)
        {
            //string to hold our return value
            string culture = string.Empty;
            if (str.IndexOf("Culture", 0) >= 0)
            {

                string[] attributes = Strings.Split(str, "=", -1, CompareMethod.Text);
                culture = attributes[1];
                if (culture.IndexOf(",") >= 0)
                {
                    culture = culture.Substring(0, culture.IndexOf(","));
                }
                _bCulture = true;
            }
            return culture;
        }
        #endregion

        #region Clear
        /// <summary>
        /// Procedure to clear the exception items when the
        /// class is initialized
        /// </summary>
        /// <remarks></remarks>
        private void Clear()
        {
	        _appVersion = "0.0";
	        _appCulture = "Unknown";
	        _methodAction = "Unknown";
	        _userName = System.Environment.UserName;
	        _machineName = System.Environment.MachineName;
	        _osVersion = System.Environment.OSVersion.ToString();
	        _bVersion = false;
	        _bCulture = false;
	        _exceptionLevel = string.Empty;
	        _exceptionType = string.Empty;
	        _stackTrace = null;
	        GetApplicationAttributes();
        }
        #endregion		
		#endregion
    }
}
