﻿using System;
using System.Drawing.Imaging;

namespace Mynx
{
    public sealed class FileHandler
    {
        private const PixelFormat PIXEL_FORMAT = PixelFormat.Format16bppRgb555;

        // Encoder info for saved JPEG image
        private ImageCodecInfo GetEncoderInfo(String mimeType)
        {
            ImageCodecInfo[] encoders = ImageCodecInfo.GetImageEncoders();
            for (int i = 0; i < encoders.Length; i++)
                if (encoders[i].MimeType == mimeType)
                    return encoders[i];
            return null;
        }

        static public string GetPath(string path)
        {
            int LastSlashIndex = path.LastIndexOf('\\');

            if (LastSlashIndex >= 0)
            {
                return path.Substring(0, LastSlashIndex);
            }
            else
            {
                return "";
            }
        }

        static public string GetFileName(string path)
        {
            int LastSlashIndex = path.LastIndexOf('\\');

            if (LastSlashIndex >= 0)
            {
                return path.Substring(LastSlashIndex + 1);
            }
            else
            {
                return path;
            }
        }

        static public string GetTitle(string path)
        {
            int LastSlashIndex = path.LastIndexOf('\\');

            if (LastSlashIndex < 0)
            {
                LastSlashIndex = 0;
            }
            else
            {
                LastSlashIndex++;
            }

            int DotIndex = path.LastIndexOf('.');

            if (DotIndex < 0)
            {
                DotIndex = path.Length;
            }

            return path.Substring(LastSlashIndex, DotIndex - LastSlashIndex);
        }

        static public string GetExtension(string path)
        {
            int LastDotIndex = path.LastIndexOf('.');

            if (LastDotIndex < 0)
            {
                return "";
            }
            else
            {
                return path.Substring(LastDotIndex);
            }
        }

        public static string ExtensionFromFormat(ImageFormat format)
        {
            if (format == ImageFormat.Jpeg) return "jpg";
            if (format == ImageFormat.Gif) return "gif";
            if (format == ImageFormat.Png) return "png";
            if (format == ImageFormat.Bmp) return "bmp";
            if (format == ImageFormat.Tiff) return "tiff";
            if (format == ImageFormat.Wmf) return "wmf";
            if (format == ImageFormat.Emf) return "emf";
            if (format == ImageFormat.Icon) return "ico";
            return null;
        }

        public static ImageFormat FormatFromExtension(string filename)
        {
            if (filename.IndexOf('.') > 0)
            {
                string ext = filename.Substring(filename.LastIndexOf('.') + 1);
                switch (ext.ToLower())
                {
                    case "jpg": return ImageFormat.Jpeg;
                    case "gif": return ImageFormat.Gif;
                    case "png": return ImageFormat.Png;
                    case "bmp": return ImageFormat.Bmp;
                    case "tif": return ImageFormat.Tiff;
                    case "wmf": return ImageFormat.Wmf;
                    case "emf": return ImageFormat.Emf;
                    case "ico": return ImageFormat.Icon;
                }
            }

            return null;
        }
    }
}
