﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Drawing2D;

namespace Mynx
{
    public partial class ImagePanel : UserControl
    {
        public ImagePanel()
        {
            InitializeComponent();

            // Set the value of the double-buffering style bits to true.
            this.SetStyle(ControlStyles.AllPaintingInWmPaint |
              ControlStyles.UserPaint | ControlStyles.ResizeRedraw |
              ControlStyles.UserPaint | ControlStyles.DoubleBuffer, true);
        }

        public int viewRectWidth { get; set; }
        public int viewRectHeight { get; set; }
        
        Point sourcePoint = new Point(0,0);
        public Point SourcePoint
        {
            get { return sourcePoint; }
            set
            {
                sourcePoint = value;
                Invalidate();
            }
        }

        float zoom = 1.0f;
        public float Zoom
        {
            get { return zoom; }
            set
            {
                if (value < 0.001f) value = 0.001f;
                zoom = value;
                Invalidate();
            }
        }

        Size canvasSize = new Size(60,40);
        public Size CanvasSize
        {
            get { return canvasSize; }
            set
            {
                canvasSize = value;
                this.Invalidate();
            }
        }

        Bitmap image;
        public Bitmap Image
        {
            get { return image; }
            set
            {
                image = value;
                Invalidate();
            }
        }

        InterpolationMode interMode = InterpolationMode.HighQualityBilinear;
        public InterpolationMode InterpolationMode
        {
            get { return interMode; }
            set { interMode = value; }
        }

        protected override void OnResize(EventArgs e)
        {
            Invalidate();
            base.OnResize(e);
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);

            //draw image
            if (image != null)
            {
                Rectangle srcRect, destRect;

                if (canvasSize.Width * Zoom < viewRectWidth && canvasSize.Height * Zoom < viewRectHeight)
                    srcRect = new Rectangle(0, 0, canvasSize.Width, canvasSize.Height);  // view all image
                else srcRect = new Rectangle(sourcePoint, new Size((int)(viewRectWidth / Zoom), (int)(viewRectHeight / Zoom))); // view a portion of image

                destRect = new Rectangle((int)(-srcRect.Width / 2), -srcRect.Height / 2, srcRect.Width, srcRect.Height); // the center of apparent image is on origin

                Matrix mx = new Matrix(); // create an identity matrix
                mx.Scale(Zoom, Zoom); // zoom image
                mx.Translate(viewRectWidth / 2.0f, viewRectHeight / 2.0f, MatrixOrder.Append); // move image to view window center

                Graphics g = e.Graphics;
                g.InterpolationMode = interMode;
                g.Transform = mx;
                g.DrawImage(image, destRect, srcRect, GraphicsUnit.Pixel);
            }
        }
    }
}
